<?php

namespace Drupal\Tests\sdc\Unit;

use Drupal\sdc\Component\ComponentMetadata;
use Drupal\Tests\UnitTestCaseTest;

/**
 * @coversDefaultClass \Drupal\sdc\Component\ComponentMetadata
 * @group sdc
 */
class ComponentMetadataTest extends UnitTestCaseTest {

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
  }

  /**
   * @dataProvider dataProviderMetadata
   */
  public function testMetadata(array $metadata_info, array $expectations) {
    $metadata = new ComponentMetadata($metadata_info, 'foo/');
    $this->assertSame($expectations['path'], $metadata->getPath());
    $this->assertSame($expectations['status'], $metadata->getStatus());
    $this->assertSame($expectations['thumbnail'], $metadata->getThumbnailPath());
    $this->assertEquals($expectations['schemas'], $metadata->getSchemas());
  }

  public function dataProviderMetadata(): array {
    return [
      [
        [
          'path' => 'foo/bar/component-name',
          'machineName' => 'component-name',
          'name' => 'Component Name',
          'libraryDependencies' => ['core/drupal'],
          'group' => 'my-group',
        ],
        [
          'path' => 'bar/component-name',
          'status' => 'READY',
          'thumbnail' => '',
          'schemas' => [
            'props' => [
              'type' => 'object',
              'additionalProperties' => FALSE,
              'required' => [],
              'properties' => [],
            ],
            'slots' => [
              'type' => 'object',
              'additionalProperties' => FALSE,
              'required' => [],
              'properties' => [],
            ],
          ],
        ],
      ],
      [
        [
          '$schema' => 'https://git.drupalcode.org/project/sdc/-/raw/1.x/src/metadata.schema.json',
          'machineName' => 'my-button',
          'path' => 'foo/my-other/path',
          'name' => 'Button',
          'description' => 'JavaScript enhanced button that tracks the number of times a user clicked it.',
          'libraryDependencies' => ['core/once'],
          'schemas' => [
            'props' => [
              'type' => 'object',
              'required' => ['text'],
              'properties' => [
                'text' => [
                  'type' => 'string',
                  'title' => 'Title',
                  'description' => 'The title for the button',
                  'minLength' => 2,
                  'examples' => ['Press', 'Submit now'],
                ],
                'iconType' => [
                  'type' => 'string',
                  'title' => 'Icon Type',
                  'enum' => [
                    'power',
                    'like',
                    'external',
                  ],
                ],
              ],
            ],
          ],
        ],
        [
          'path' => 'my-other/path',
          'status' => 'READY',
          'thumbnail' => '',
          'additionalProperties' => FALSE,
          'schemas' => [
            'props' => [
              'type' => 'object',
              'required' => ['text'],
              'additionalProperties' => FALSE,
              'properties' => [
                'text' => [
                  'type' => ['string'],
                  'title' => 'Title',
                  'description' => 'The title for the button',
                  'minLength' => 2,
                  'examples' => ['Press', 'Submit now'],
                ],
                'iconType' => [
                  'type' => ['string'],
                  'title' => 'Icon Type',
                  'enum' => [
                    'power',
                    'like',
                    'external',
                  ],
                ],
              ],
            ],
            'slots' => [
              'type' => 'object',
              'additionalProperties' => FALSE,
              'required' => [],
              'properties' => [],
            ],
          ],
        ],
      ],
    ];
  }

}
